'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) {
    return;
  }

  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var highlightClassName = 'react-highlight';
var styles = "\n  .".concat(highlightClassName, " {\n    background-color: #f8ffb4;\n    transition: all 0.5s ease-in-out;\n  }\n");
var styleSheet = document.createElement('style');
styleSheet.type = 'text/css';
styleSheet.innerText = styles;
document.head.appendChild(styleSheet);

var ChangeHighlight = (function (_ref) {
  var children = _ref.children,
      _ref$showAfter = _ref.showAfter,
      showAfter = _ref$showAfter === void 0 ? 500 : _ref$showAfter,
      _ref$hideAfter = _ref.hideAfter,
      hideAfter = _ref$hideAfter === void 0 ? 2500 : _ref$hideAfter,
      _ref$containerClassNa = _ref.containerClassName,
      containerClassName = _ref$containerClassNa === void 0 ? '' : _ref$containerClassNa,
      _ref$highlightStyle = _ref.highlightStyle,
      highlightStyle = _ref$highlightStyle === void 0 ? highlightClassName : _ref$highlightStyle,
      _ref$disabled = _ref.disabled,
      disabled = _ref$disabled === void 0 ? false : _ref$disabled;

  var _useState = React.useState(),
      _useState2 = _slicedToArray(_useState, 2),
      oldChildren = _useState2[0],
      setOldChildren = _useState2[1];

  var _useState3 = React.useState([]),
      _useState4 = _slicedToArray(_useState3, 2),
      listOfHighlightedElements = _useState4[0],
      setListOfHighlightedElements = _useState4[1];

  var isInitialMount = React.useRef(true);

  var showHighlight = function showHighlight(element) {
    var showAfter = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : showAfter;
    var hideAfter = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : hideAfter;
    var classNames = element.ref.current.className;
    var isHighlighted = element.ref.current.className.includes(highlightStyle);

    if (!isHighlighted) {
      setTimeout(function () {
        element.ref.current.className += ' ' + highlightStyle;
      }, showAfter);
    } else {
      element.ref.current.className = classNames.indexOf(highlightStyle) && classNames.substr(0, classNames.indexOf(highlightStyle)).trim();
      showHighlight(element, showAfter, hideAfter);
    }
  };

  var hideHighlight = function hideHighlight(element, hideAfter, oneElement) {
    setTimeout(function () {
      var classNames = element.ref.current.className;

      if (classNames.indexOf(highlightStyle) > -1) {
        element.ref.current.className = classNames.substr(0, classNames.indexOf(highlightStyle)).trim();
      }

      oneElement.highlighted = true;
    }, hideAfter);
  };

  var checkChangedChildren = function checkChangedChildren(children, oldChildren) {
    React__default.Children.forEach(oldChildren, function (oldChild, index1) {
      React__default.Children.forEach(children, function (newChild, index2) {
        if (index1 === index2) {
          if (oldChild.ref && newChild.props.children !== oldChild.props.children) {
            if (newChild.props.children) {
              setListOfHighlightedElements([].concat(_toConsumableArray(listOfHighlightedElements), [{
                element: newChild,
                hideAfter: hideAfter
              }]));
              showHighlight(newChild, showAfter, hideAfter);
            }

            setOldChildren(children);
          }
        }
      });
    });
  };

  React.useEffect(function () {
    if (disabled) return;

    if (isInitialMount.current) {
      isInitialMount.current = false;
      setOldChildren(children);
    } else {
      checkChangedChildren(children, oldChildren);
    }
  });

  var pickNotHighlightedElement = function pickNotHighlightedElement(allChanged) {
    var idx = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var oneElement;

    if (idx < allChanged.length) {
      oneElement = listOfHighlightedElements[idx];

      if (oneElement.highlighted) {
        pickNotHighlightedElement(allChanged, idx++);
      }
    }

    return oneElement;
  };

  React.useEffect(function () {
    var length = listOfHighlightedElements.length;

    if (length) {
      var oneElement = pickNotHighlightedElement(listOfHighlightedElements);

      if (oneElement) {
        var element = oneElement.element,
            _hideAfter = oneElement.hideAfter;
        hideHighlight(element, _hideAfter, oneElement);
      }
    }
  }, [listOfHighlightedElements]);
  return React__default.createElement("div", {
    className: containerClassName
  }, children);
});

module.exports = ChangeHighlight;
